package cn.com.qmth.stmms.api.controller.admin;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.aliyun.oss.common.utils.BinaryUtil;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.tools.io.ZipReader;

import cn.com.qmth.stmms.admin.dto.ObjectiveQuestionDTO;
import cn.com.qmth.stmms.admin.dto.QuestionDTO;
import cn.com.qmth.stmms.admin.dto.SubjectQuestionDTO;
import cn.com.qmth.stmms.admin.dto.SubjectiveQuestionDTO;
import cn.com.qmth.stmms.admin.thread.ScoreCalculateThread;
import cn.com.qmth.stmms.admin.thread.ScoreReportObjectiveThread;
import cn.com.qmth.stmms.admin.thread.ScoreReportThread;
import cn.com.qmth.stmms.admin.vo.StructFile;
import cn.com.qmth.stmms.admin.vo.StructQuestion;
import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.common.domain.card.AnswerCardSubjectFile;
import cn.com.qmth.stmms.biz.common.domain.card.AnswerCardSubjectItem;
import cn.com.qmth.stmms.biz.common.domain.card.CardFile;
import cn.com.qmth.stmms.biz.exam.bean.ExamVo;
import cn.com.qmth.stmms.biz.exam.bean.ResultMessage;
import cn.com.qmth.stmms.biz.exam.bean.SubjectSplit;
import cn.com.qmth.stmms.biz.exam.model.AnswerCard;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.model.SelectiveGroup;
import cn.com.qmth.stmms.biz.exam.service.AnswerCardService;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.InspectHistoryService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.exam.service.SelectiveGroupService;
import cn.com.qmth.stmms.biz.exam.service.query.ExamQuestionSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.query.ExamSubjectSearchQuery;
import cn.com.qmth.stmms.biz.file.enums.FormatType;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.report.service.ReportService;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.CardSource;
import cn.com.qmth.stmms.common.enums.ExamStatus;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.MarkStatus;
import cn.com.qmth.stmms.common.enums.ObjectiveStatus;
import cn.com.qmth.stmms.common.enums.SubjectiveStatus;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.ImportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONObject;

@Api(tags = "科目管理")
@Controller("adminSubjectController")
@RequestMapping("/api/admin/subject")
public class SubjectController extends BaseApiController {

    protected static final Logger log = LoggerFactory.getLogger(SubjectController.class);

    private static final String[] SUBJECTIVE_EXCEL_HEADER = new String[] { "科目代码*", "科目名称", "大题名称*", "题目昵称",
            "大题号(只能用小写数字)*", "小题号(只能用小写数字)*", "小题满分*", "给分次数", "间隔分*", "评卷分组(只能用小写数字)*", "图片序号(用英文逗号分割)",
            "仲裁方式(0-分组,1-小题)", "双评比例(0~1)", "仲裁阀值", "合分策略(1-平均，2-最高，3-最低)", "评卷模式(common-普通，track-轨迹)", "试评数量(0-跳过试评)",
            "选做题数量" };

    private static final String[] OBJECTIVE_EXCEL_HEADER = new String[] { "科目代码", "科目名称", "试卷类型", "大题名称", "题目昵称",
            "大题号(只能用小写数字)", "小题号(只能用小写数字)", "标准答案", "小题满分", "题型(1-单选，2-多选，3-判断)" };

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamService examService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private FileService fileService;

    @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @Autowired
    private LockService lockService;

    @Autowired
    private ReportService reportService;

    @Autowired
    private SelectiveGroupService selectiveGroupService;

    @Autowired
    private AnswerCardService answerCardService;

    @Autowired
    private InspectHistoryService inspectHistoryService;

    @Autowired
    private MarkerService markerService;

    @Value("${file.temp}")
    private String tempFile;

    @ApiOperation(value = "科目下拉列表")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    @ResponseBody
    public List<ExamSubject> list() {
        int examId = getSessionExamId();
        ApiUser wu = getApiUser();
        if (wu.isMarker()) {
            List<Marker> list = markerService.findByExamAndUserId(examId, wu.getUser().getId());
            Map<String, ExamSubject> map = new HashMap<>();
            for (Marker marker : list) {
                MarkGroup group = groupService.findOne(examId, marker.getSubjectCode(), marker.getGroupNumber());
                if (map.get(marker.getSubjectCode()) == null && marker.isEnable()
                        && group.getStatus() != MarkStatus.FINISH) {
                    ExamSubject subject = subjectService.find(marker.getExamId(), marker.getSubjectCode());
                    map.put(marker.getSubjectCode(), subject);
                }
            }
            return new ArrayList<>(map.values());
        }
        return getExamSubject(examId, wu);
    }

    @ApiOperation(value = "科目层次下拉列表")
    @RequestMapping(value = "/level/list", method = RequestMethod.POST)
    @ResponseBody
    public List<String> levelList(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        return subjectService.listLevel(examId);
    }

    @ApiOperation(value = "科目专业下拉列表")
    @RequestMapping(value = "/category/list", method = RequestMethod.POST)
    @ResponseBody
    public List<String> categoryList(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        return subjectService.listCategory(examId);
    }

    @ApiOperation(value = "科目查询")
    @RequestMapping(value = "/query", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<ExamSubject> list(HttpServletRequest request, ExamSubjectSearchQuery query) {
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        query = subjectService.findByQuery(query);
        for (ExamSubject subject : query.getResult()) {
            List<String> paperTypes = questionService.getPaperType(examId, subject.getCode());
            subject.setPaperType(StringUtils.join(paperTypes, ","));
            subject.setPaperAnswerUrl(fileService);
            subject.setLocked(lockService.isLocked(LockType.SCORE_CALCULATE, examId, subject.getCode()));
        }
        return PageUtil.of(query);
    }

    @ApiOperation(value = "获取指定科目")
    @RequestMapping(value = "/find", method = RequestMethod.POST)
    @ResponseBody
    public ExamSubject find(HttpServletRequest request, @RequestParam Integer examId, @RequestParam String code) {
        return subjectService.find(examId, code);
    }

    @ApiOperation(value = "科目修改")
    @Logging(menu = "科目设置修改", type = LogType.UPDATE)
    @RequestMapping(value = "/update", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage save(HttpServletRequest request, ExamSubject subject) {
        ExamSubject previous = subjectService.find(getSessionExamId(request), subject.getCode());
        if (previous == null) {
            throw new StatusException("科目不存在");
        }
        previous.setSheetConfig(subject.getSheetConfig());
        previous.setSasConfig(subject.getSasConfig());
        previous.setEnableSplit(subject.getEnableSplit());
        previous.setAutoScroll(subject.getAutoScroll());
        previous.setDisplayQuestionName(
                subject.getDisplayQuestionName() == null ? false : subject.getDisplayQuestionName());
        subjectService.save(previous);
        return resultOk();
    }

    @ApiOperation(value = "科目总分统计")
    @RequestMapping(value = "/statistic", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage statistic(HttpServletRequest request) {
        int examId = getSessionExamId(request);
        List<Double> totalScoreList = subjectService.distinctTotalScore(examId);
        List<Long> subjectCountList = new ArrayList<Long>();
        for (Double totalScore : totalScoreList) {
            ExamSubjectSearchQuery querys = new ExamSubjectSearchQuery();
            querys.setExamId(examId);
            querys.setTotalScore(totalScore);
            subjectCountList.add(subjectService.countByQuery(querys));
        }
        return resultOk();
    }

    @ApiOperation(value = "卷型下拉列表")
    @RequestMapping(value = "/getPaperType", method = RequestMethod.POST)
    @ResponseBody
    public List<String> query(HttpServletRequest request, @RequestParam String subjectCode) {
        int examId = getSessionExamId(request);
        List<String> list = this.questionService.getPaperType(examId, subjectCode);
        return list;
    }

    @ApiOperation(value = "客or主观题导入模版")
    @RequestMapping(value = "/template", method = RequestMethod.POST)
    public void importTemplate(HttpServletResponse response, @RequestParam Boolean objective) {
        try {
            String fileName = objective ? "客观题导入模板.xlsx" : "主观题导入模板.xlsx";
            String title = objective ? "客观题数据" : "主观题数据";
            List<QuestionDTO> list = new LinkedList<QuestionDTO>();
            list.add(objective ? new ObjectiveQuestionDTO() : new SubjectiveQuestionDTO());
            new ExportExcel(title, objective ? ObjectiveQuestionDTO.class : SubjectiveQuestionDTO.class, 2)
                    .setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
            throw new StatusException("导入模板下载失败！失败信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "客观题统分")
    @Logging(menu = "客观题统分", type = LogType.UPDATE)
    @RequestMapping(value = "/calculate", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage calculate(HttpServletRequest request, @RequestParam(required = false) String subjectCode) {
        ApiUser wu = RequestUtils.getApiUser(request);
        Exam exam = examService.findById(getSessionExamId(request));
        if (exam == null || !exam.getSchoolId().equals(wu.getUser().getSchoolId())) {
            throw new StatusException("请选择正确的考试");
        } else {
            Set<String> subjectSet = new HashSet<String>();
            // 整个考试加锁
            boolean examLock = false;
            if (subjectCode == null) {
                subjectSet = getCalculateSubjectCodes(exam.getId());
                examLock = true;
            } else if (!lockService.isLocked(LockType.SCORE_CALCULATE, exam.getId(), subjectCode)) {
                subjectSet.add(subjectCode);
            }
            if (!subjectSet.isEmpty()) {
                ScoreCalculateThread thread = new ScoreCalculateThread(exam.getId(), subjectSet, lockService,
                        studentService, questionService, examService, subjectService, fileService, examLock);
                taskExecutor.submit(thread);
            }
        }
        return resultOk();
    }

    private Set<String> getCalculateSubjectCodes(Integer examId) {
        List<ExamSubject> list = subjectService.list(examId);
        Set<String> set = new HashSet<>();
        for (ExamSubject subject : list) {
            if (!lockService.isLocked(LockType.SCORE_CALCULATE, subject.getExamId(), subject.getCode())) {
                set.add(subject.getCode());
            }
        }
        return set;
    }

    @ApiOperation(value = "导出客or主观题")
    @Logging(menu = "导出客/主观题", type = LogType.EXPORT)
    @RequestMapping(value = "/export", method = RequestMethod.POST)
    public void exportFile(HttpServletRequest request, HttpServletResponse response, @RequestParam Boolean objective) {
        int examId = getSessionExamId(request);

        Map<String, ExamSubject> subjectMap = new HashMap<String, ExamSubject>();
        List<ExamSubject> subjectList = subjectService.list(examId);
        for (ExamSubject subject : subjectList) {
            subjectMap.put(subject.getCode(), subject);
        }

        Map<String, MarkGroup> groupMap = new HashMap<String, MarkGroup>();
        List<MarkGroup> groupList = groupService.findByExam(examId);
        for (MarkGroup group : groupList) {
            groupMap.put(group.getSubjectCode() + "_" + group.getNumber(), group);
        }
        List<QuestionDTO> list = new LinkedList<QuestionDTO>();
        ExamQuestionSearchQuery query = new ExamQuestionSearchQuery();
        query.setExamId(examId);
        query.setObjective(objective);
        query.setSort(new Sort(Sort.Direction.ASC, "subjectCode", "paperType", "mainNumber", "subNumber"));
        query.setPageNumber(1);
        query.setPageSize(Integer.MAX_VALUE);
        query = questionService.findByQuery(query);
        for (ExamQuestion q : query.getResult()) {
            list.add(objective ? new ObjectiveQuestionDTO(q, subjectMap.get(q.getSubjectCode()))
                    : new SubjectiveQuestionDTO(q, subjectMap.get(q.getSubjectCode()),
                            groupMap.get(q.getSubjectCode() + "_" + q.getGroupNumber())));
        }
        try {
            String fileName = objective ? "客观题数据.xlsx" : "主观题数据.xlsx";
            String title = objective ? "客观题数据" : "主观题数据";
            new ExportExcel(title, objective ? ObjectiveQuestionDTO.class : SubjectiveQuestionDTO.class, 2)
                    .setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
            throw new StatusException("导出数据失败！失败信息：" + e.getMessage());
        }
    }

    @ApiOperation(value = "导入客or主观题")
    @Logging(menu = "导入客/主观题", type = LogType.IMPORT_FILE)
    @RequestMapping(value = "/import", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject importFile(HttpServletRequest request, MultipartFile file, @RequestParam Boolean objective) {
        int examId = getSessionExamId(request);

        List<String> error = new LinkedList<String>();
        Map<String, SubjectQuestionDTO> map = parseQuestion(file, examId, objective, error);
        if (error.size() > 0) {
            return result(StringUtils.join(error, "<br\\>"));
        }
        int success = 0;
        for (SubjectQuestionDTO dto : map.values()) {
            // 每次导入都需要重新统分
            examService.updateObjectiveStatus(examId, ObjectiveStatus.WAITING);
            ExamSubject subject = subjectService.find(examId, dto.getSubjectCode());
            if (subject != null) {
                Map<Integer, String> titleMap = new HashMap<>();
                List<ExamQuestion> current = questionService.findByExamAndSubjectAndObjective(examId, subject.getCode(),
                        objective);
                for (ExamQuestion question : current) {
                    titleMap.put(question.getMainNumber(), question.getMainTitle());
                }
                if (dto.validate(error, titleMap)) {
                    if (objective) {
                        questionService.deleteByExamAndSubjectAndObjective(examId, dto.getSubjectCode(), objective);
                        for (ExamQuestion question : dto.getQuestionList()) {
                            questionService.save(question);
                        }
                        subjectService.updateScore(examId, subject.getCode(), objective, dto.getTotalScore());
                    } else {
                        int questionCount = 0;
                        List<SelectiveGroup> list = selectiveGroupService.findByExamIdAndSubjectCode(examId,
                                subject.getCode());
                        Map<Integer, SelectiveGroup> selectiveMap = new HashMap<Integer, SelectiveGroup>();
                        for (SelectiveGroup selectiveGroup : list) {
                            selectiveMap.put(selectiveGroup.getMainNumber(), selectiveGroup);
                        }
                        for (ExamQuestion question : dto.getQuestionList()) {
                            ExamQuestion old = questionService
                                    .findByExamAndSubjectAndObjectiveAndMainNumberAndSubNumber(examId,
                                            subject.getCode(), objective, question.getMainNumber(),
                                            question.getSubNumber());
                            if (old != null) {
                                if (old.getGroupNumber() != null) {
                                    error.add("[" + subject.getCode() + "] 大题号" + question.getMainNumber() + " 小题号"
                                            + question.getSubNumber() + "已有分组");
                                    continue;
                                }
                                if (selectiveMap.containsKey(question.getMainNumber())) {
                                    error.add("[" + subject.getCode() + "] 大题号" + question.getMainNumber() + " 小题号"
                                            + question.getSubNumber() + "已有选做题分组");
                                    continue;
                                }
                                old.setName(question.getName());
                                old.setTotalScore(question.getTotalScore());
                                old.setIntervalScore(question.getIntervalScore());
                                old.setTrackCount(question.getTrackCount());
                                questionService.save(old);
                            } else {
                                question.setGroupNumber(null);
                                questionService.save(question);
                                questionCount++;
                            }
                        }
                        if (questionCount > 0) {
                            studentService.updateSubjectiveStatusAndScore(examId, subject.getCode(),
                                    SubjectiveStatus.UNMARK, 0, null);
                            inspectHistoryService.deleteByExamIdAndSubjectCode(examId, subject.getCode());
                        }
                        subjectService.updateScore(examId, subject.getCode(), objective,
                                questionService.sumTotalScore(examId, subject.getCode(), objective));
                    }
                    success++;
                }
            } else {
                error.add("[" + dto.getSubjectCode() + "] 科目代码不存在；");
            }
        }
        error.add(0, success + "个科目导入成功；");
        if (success > 0) {
            subjectService.updateTotalScore(examId);
        }
        RequestUtils.setLog(request, success + "个科目导入成功；");
        return result(StringUtils.join(error, "<br\\>"));
    }

    private Map<String, SubjectQuestionDTO> parseQuestion(MultipartFile file, int examId, boolean objective,
            List<String> error) {
        Map<String, SubjectQuestionDTO> map = new HashMap<String, SubjectQuestionDTO>();
        try {
            ImportExcel excel = new ImportExcel(file, 1, 0);
            Class<? extends QuestionDTO> clazz = objective ? ObjectiveQuestionDTO.class : SubjectiveQuestionDTO.class;
            if (!objective && !Arrays.equals(SUBJECTIVE_EXCEL_HEADER, excel.getHeaderName())) {
                error.add("Excel表头错误");
                return map;
            }
            if (objective && !Arrays.equals(OBJECTIVE_EXCEL_HEADER, excel.getHeaderName())) {
                error.add("Excel表头错误");
                return map;
            }
            List<? extends QuestionDTO> list = excel.getDataList(clazz);
            if (list != null) {
                for (QuestionDTO dto : list) {
                    String subjectCode = StringUtils.trimToNull(dto.getSubjectCode());
                    if (subjectCode != null) {
                        SubjectQuestionDTO subject = map.get(subjectCode);
                        if (subject == null) {
                            subject = new SubjectQuestionDTO(examId, subjectCode, objective);
                            map.put(subjectCode, subject);
                        }
                        subject.addQuestion(dto.transform());
                    }
                }
            }
        } catch (Exception e) {
            log.error("parse question excel error", e);
            error.add("Excel文件解析失败");
        }
        return map;
    }

    @ApiOperation(value = "导入主观题评卷分组")
    @Logging(menu = "导入主观题分组", type = LogType.IMPORT_FILE)
    @RequestMapping(value = "/importGroup", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject importGroupFile(HttpServletRequest request, MultipartFile file) {
        int examId = getSessionExamId(request);
        List<String> error = new LinkedList<String>();
        Map<String, SubjectQuestionDTO> map = parseQuestion(file, examId, false, error);
        if (error.size() > 0) {
            return result(StringUtils.join(error, " "));
        }
        int success = 0;
        for (SubjectQuestionDTO dto : map.values()) {
            ExamSubject subject = subjectService.find(examId, dto.getSubjectCode());
            if (subject != null) {
                // 验证是否存在分组
                List<MarkGroup> groups = groupService.findByExamAndSubject(examId, subject.getCode());
                if (groups != null && groups.size() > 0) {
                    error.add("[" + subject.getCode() + "] 已存在分组");
                    continue;
                }
                if (subject.isSelective()) {
                    error.add("[" + subject.getCode() + "] 已存在选做题分组");
                    continue;
                }
                Map<Integer, String> titleMap = new HashMap<>();
                List<ExamQuestion> current = questionService.findByExamAndSubjectAndObjective(examId, subject.getCode(),
                        false);
                for (ExamQuestion question : current) {
                    titleMap.put(question.getMainNumber(), question.getMainTitle());
                }
                if (dto.validate(error, titleMap) && dto.validateGroupNumber(error)
                        && dto.getQuestionList().size() == current.size()) {
                    int successGroup = 0;
                    for (MarkGroup group : dto.getGroups().values()) {
                        if (group.getImportQuestionList() != null) {
                            boolean validate = true;
                            List<ExamQuestion> questionGroup = new ArrayList<ExamQuestion>();
                            for (ExamQuestion question : group.getImportQuestionList()) {
                                ExamQuestion old = questionService
                                        .findByExamAndSubjectAndObjectiveAndMainNumberAndSubNumber(examId,
                                                subject.getCode(), false, question.getMainNumber(),
                                                question.getSubNumber());
                                if (old == null) {
                                    error.add("[" + group.getSubjectCode() + "_" + group.getNumber() + "] 分组有小题不存在");
                                    validate = false;
                                    break;
                                }
                                old.setGroupNumber(group.getNumber());
                                old.setArbitrateThreshold(question.getArbitrateThreshold());
                                questionGroup.add(old);
                            }
                            // 校验通过小题才保存
                            if (validate && !questionGroup.isEmpty()) {
                                questionService.save(questionGroup);
                            }
                            // 有题目的分组才保存
                            if (questionService.countByExamAndSubjectAndObjectiveAndGroupNumber(examId,
                                    group.getSubjectCode(), false, group.getNumber()) != 0) {
                                BigDecimal totalScore = BigDecimal.ZERO;
                                for (ExamQuestion q : questionGroup) {
                                    totalScore = totalScore.add(BigDecimal.valueOf(q.getTotalScore()));
                                }
                                group.setTotalScore(totalScore.doubleValue());
                                groupService.save(group);
                                studentService.updateSubjectiveStatusAndScore(examId, group.getSubjectCode(),
                                        SubjectiveStatus.UNMARK, 0, null);
                                inspectHistoryService.deleteByExamIdAndSubjectCode(examId, subject.getCode());
                                successGroup++;
                            }
                        }
                    }
                    if (successGroup > 0) {
                        success++;
                        subjectService.updateTrialCount(examId, subject.getCode(), dto.getTrialCount());
                    }
                }
            } else {
                error.add("[" + dto.getSubjectCode() + "] 科目代码不存在；");
            }
        }
        error.add(0, success + "个科目导入成功；");
        RequestUtils.setLog(request, success + "个科目导入成功；");
        return result(StringUtils.join(error, " "));
    }

    @ApiOperation(value = "科目成绩分析计算")
    @Logging(menu = "科目成绩分析计算", type = LogType.UPDATE)
    @RequestMapping(value = "/report", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage report(HttpServletRequest request, @RequestParam(required = false) String subjectCode) {
        ApiUser wu = RequestUtils.getApiUser(request);
        Exam exam = examService.findById(getSessionExamId(request));
        if (exam == null || !exam.getSchoolId().equals(wu.getUser().getSchoolId())) {
            throw new StatusException("请选择正确的考试");
        } else {
            Set<String> subjectSet = new HashSet<String>();
            // 整个考试加锁
            boolean examLock = false;
            if (subjectCode == null) {
                examLock = true;
                List<ExamSubject> list = subjectService.list(exam.getId());
                for (ExamSubject subject : list) {
                    if (!lockService.isLocked(LockType.SCORE_CALCULATE, subject.getExamId(), subject.getCode())) {
                        subjectSet.add(subject.getCode());
                    }
                }
            } else if (!lockService.isLocked(LockType.SCORE_CALCULATE, exam.getId(), subjectCode)) {
                subjectSet.add(subjectCode);
            }
            if (!subjectSet.isEmpty()) {
                Map<Integer, Set<String>> map = new HashMap<Integer, Set<String>>();
                map.put(exam.getId(), subjectSet);
                ScoreReportThread thread = new ScoreReportThread(map, reportService, examLock);
                taskExecutor.submit(thread);
            }
        }
        return resultOk();
    }

    @ApiOperation(value = "客观题分析计算")
    @Logging(menu = "客观题分析计算", type = LogType.UPDATE)
    @RequestMapping(value = "/report/objective", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage reportObjective(HttpServletRequest request) {
        ApiUser wu = RequestUtils.getApiUser(request);
        Exam exam = examService.findById(getSessionExamId(request));
        if (exam == null || !exam.getSchoolId().equals(wu.getUser().getSchoolId())) {
            throw new StatusException("请选择正确的考试");
        } else {
            Set<String> subjectSet = new HashSet<String>();
            // 整个考试加锁
            boolean examLock = true;
            List<ExamSubject> list = subjectService.list(exam.getId());
            for (ExamSubject subject : list) {
                if (!lockService.isLocked(LockType.SCORE_CALCULATE, subject.getExamId(), subject.getCode())) {
                    subjectSet.add(subject.getCode());
                }
            }
            if (!subjectSet.isEmpty()) {
                Map<Integer, Set<String>> map = new HashMap<Integer, Set<String>>();
                map.put(exam.getId(), subjectSet);
                ScoreReportObjectiveThread thread = new ScoreReportObjectiveThread(map, reportService, examLock);
                taskExecutor.submit(thread);
            }
        }
        return resultOk();
    }

    @ApiOperation(value = "导入数据包")
    @Logging(menu = "导入数据包", type = LogType.IMPORT_FILE)
    @RequestMapping(value = "/importData", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage importData(HttpServletRequest request, MultipartFile file) {
        int examId = getSessionExamId(request);
        List<String> error = new ArrayList<String>();
        try {
            byte[] data = IOUtils.toByteArray(file.getInputStream());
            File target = new File(tempFile + File.separator + "temp.zip");
            target.getParentFile().mkdirs();
            FileOutputStream ous = new FileOutputStream(target);
            ous.write(data);
            IOUtils.closeQuietly(ous);

            ZipReader zipReader = new ZipReader(target);
            for (String subjectCode : zipReader.list(false)) {
                try {
                    if (subjectCode.equalsIgnoreCase("cards")) {
                        error = this.parseCard(examId, error, zipReader);
                    } else {
                        error = this.parseSubject(examId, subjectCode, error, zipReader);
                    }
                } catch (Exception e) {
                    log.error("parse zip data error", e);
                    error.add("[" + subjectCode + "] 科目代码导入失败；");
                }
            }
            subjectService.updateCardType(examId);
            target.delete();
        } catch (Exception e) {
            log.error("parse question excel error", e);
            error.add("zip文件解析失败");
        }
        if (error.size() > 0) {
            ResultMessage resultMessage = new ResultMessage();
            resultMessage.setMessage(StringUtils.join(error, " "));
            resultMessage.setSuccess(false);
            return resultMessage;
        }
        return resultOk();
    }

    private List<String> parseCard(int examId, List<String> error, ZipReader zipReader) throws IOException {
        InputStream ins = zipReader.read("cards", "description.json");
        List<AnswerCardSubjectFile> courseList = AnswerCardSubjectFile.parse(ins);
        for (AnswerCardSubjectFile answerCardSubjectFile : courseList) {
            try {
                InputStream cardIns = zipReader.read("cards", answerCardSubjectFile.getCode() + ".json");
                CardFile cardFile = CardFile.parse(cardIns);
                if (cardFile.getPages().size() != 1 && cardFile.getPages().size() % 2 != 0) {
                    error.add("[" + answerCardSubjectFile.getCode() + "] 科目代码卡格式解析有误；");
                    return error;
                }
                byte[] out = cardFile.output();
                String md5 = BinaryUtil.encodeMD5(out);
                AnswerCard card = answerCardService.findByExamIdAndCode(examId, answerCardSubjectFile.getCode());
                if (card == null) {
                    card = new AnswerCard();
                    card.setExamId(examId);
                    Integer number = answerCardService.findMaxNumberByExamId(examId) + 1;
                    card.setNumber(number);
                    card.setCode(answerCardSubjectFile.getCode());
                }
                card.setMd5(md5);
                card.setNeedAdapte(true);
                card.setSliceConfig(cardFile.getSliceConfig().toString());
                card.setSinglePage(cardFile.getPages().size() == 1);
                card.setPaperCount(card.getSinglePage() ? 1 : cardFile.getPages().size() / 2);
                card.setSource(CardSource.WEB);
                card.setUpdateTime(new Date());
                List<String> asList = new ArrayList<String>();
                List<ExamSubject> subjects = new ArrayList<ExamSubject>();
                for (AnswerCardSubjectItem item : answerCardSubjectFile.getCourses()) {
                    ExamSubject subject = subjectService.find(examId, item.getCode());
                    if (subject == null) {
                        error.add("[" + item.getCode() + "] 科目代码不存在；");
                        return error;
                    }
                    if (answerCardService.hasSubject(examId, item.getCode(), item.getCode())) {
                        error.add("[" + item.getCode() + "] 科目代码已经绑定卡格式；");
                        return error;
                    } else {
                        asList.add(item.getCode());
                        subject.setSliceConfig(cardFile.getSliceConfig().toString());
                        subject.setCardType(FormatType.JSON);
                        subjects.add(subject);
                    }
                }
                answerCardService.save(card, asList);
                fileService.uploadAnswerCard(new ByteArrayInputStream(out), md5, examId, card.getNumber());
                for (ExamSubject examSubject : subjects) {
                    subjectService.save(examSubject);
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.error("parse zip data error", e);
                error.add("[" + answerCardSubjectFile.getCode() + "] 科目代码卡格式解析有误；");
                return error;
            }
        }
        return error;
    }

    private List<String> parseSubject(int examId, String subjectCode, List<String> error, ZipReader zipReader)
            throws Exception {
        ExamSubject subject = subjectService.find(examId, subjectCode);
        if (subject == null) {
            error.add("[" + subjectCode + "] 科目代码不存在；");
            return error;
        }
        if (subject.getTotalScore() != 0) {
            error.add("[" + subjectCode + "] 科目代码已经存在题目；");
            return error;
        }
        InputStream cardIns = zipReader.read(subjectCode, "card.json");
        CardFile cardFile = CardFile.parse(cardIns);
        if (cardFile.getPages().size() != 1 && cardFile.getPages().size() % 2 != 0) {
            error.add("[" + subjectCode + "] 科目代码卡格式解析有误；");
            return error;
        }
        byte[] out = cardFile.output();
        String md5 = BinaryUtil.encodeMD5(out);
        AnswerCard card = answerCardService.findByExamIdAndSubjectCode(examId, subject.getCode(), CardSource.WEB);
        if (card == null) {
            card = new AnswerCard();
            card.setExamId(examId);
            Integer number = answerCardService.findMaxNumberByExamId(examId) + 1;
            card.setNumber(number);
        }
        card.setMd5(md5);
        card.setNeedAdapte(true);
        card.setSliceConfig(cardFile.getSliceConfig().toString());
        card.setSinglePage(cardFile.getPages().size() == 1);
        card.setPaperCount(card.getSinglePage() ? 1 : cardFile.getPages().size() / 2);
        card.setSource(CardSource.WEB);
        card.setUpdateTime(new Date());
        answerCardService.save(card, subjectCode);

        fileService.uploadAnswerCard(new ByteArrayInputStream(out), md5, examId, card.getNumber());
        subject.setSliceConfig(cardFile.getSliceConfig().toString());
        subject.setCardType(FormatType.JSON);
        subjectService.save(subject);

        InputStream structIns = zipReader.read(subjectCode, "struct.json");
        StructFile structFile = StructFile.parse(structIns);
        if (!structFile.getObjective().isEmpty()) {
            List<ExamQuestion> oList = new ArrayList<ExamQuestion>();
            for (StructQuestion structQuestion : structFile.getObjective()) {
                ExamQuestion q = structQuestion.transform();
                q.setPaperType(structFile.getPaperType());
                q.setExamId(examId);
                q.setSubjectCode(subject.getCode());
                q.setObjective(true);
                oList.add(q);
            }
            questionService.save(oList);
            examService.updateObjectiveStatus(examId, ObjectiveStatus.WAITING);
            subjectService.updateScore(examId, subject.getCode(), true,
                    questionService.sumTotalScore(examId, subject.getCode(), true));
        }
        if (!structFile.getSubjective().isEmpty()) {
            List<ExamQuestion> sList = new ArrayList<ExamQuestion>();
            Map<Integer, Double> mainMap = new HashMap<Integer, Double>();
            for (StructQuestion structQuestion : structFile.getSubjective()) {
                ExamQuestion q = structQuestion.transform();
                q.setPaperType(structFile.getPaperType());
                q.setExamId(examId);
                q.setSubjectCode(subject.getCode());
                q.setObjective(false);
                q.setGroupNumber(q.getMainNumber());
                sList.add(q);
                Double totalScore = mainMap.get(q.getMainNumber());
                if (totalScore == null) {
                    mainMap.put(q.getMainNumber(), q.getTotalScore());
                } else {
                    mainMap.put(q.getMainNumber(), totalScore + q.getTotalScore());
                }
            }
            questionService.save(sList);
            for (Integer number : mainMap.keySet()) {
                MarkGroup group = new MarkGroup(examId, subject.getCode(), number, null, mainMap.get(number), null,
                        null, null, null, null, 0, false, false, null, false);
                group.setPicList(cardFile.getMarkConfigAdd(cardFile.getMarkConfig(number), 0.015));
                groupService.save(group);
            }
            subjectService.updateScore(examId, subject.getCode(), false,
                    questionService.sumTotalScore(examId, subject.getCode(), false));
        }
        return error;
    }

    @ApiOperation(value = "拆分科目-考试列表")
    @RequestMapping(value = "split/exam", method = RequestMethod.POST)
    @ResponseBody
    public List<ExamVo> splitExam(@RequestParam Integer schoolId) {
        List<Exam> examList = examService.findBySchoolId(schoolId);
        List<ExamVo> ret = new ArrayList<>();
        if (!CollectionUtils.isEmpty(examList)) {
            for (Exam e : examList) {
                if (!ExamStatus.FINISH.equals(e.getStatus())) {
                    ret.add(ExamVo.of(e));
                }
            }
        }
        return ret;
    }

    @ApiOperation(value = "拆分科目-导入")
    @RequestMapping(value = "split/save", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage splitSave(MultipartFile file, @RequestParam Integer schoolId, @RequestParam Integer examId) {
        try {
            ImportExcel ei = new ImportExcel(file, 1, 0);
            List<SubjectSplit> list = ei.getDataList(SubjectSplit.class);
            String errMsg = subjectService.split(schoolId, examId, list);
            if (!StringUtils.isBlank(errMsg)) {
                ApiUser wu = getApiUser();
                File errFile = new File(getSubjectSplitErrFile(wu.getId()));
                errFile.getParentFile().mkdirs();
                if (errFile.exists()) {
                    errFile.delete();
                }
                FileUtils.write(errFile, errMsg, "utf-8");
                throw new StatusException("导入信息有误，请查看error.txt文件");
            }
        } catch (Exception e) {
            log.error("导入失败", e);
            throw new StatusException("导入失败：" + e.getMessage(), e);
        }
        return resultOk();
    }

    @ApiOperation(value = "拆分科目-错误信息")
    @RequestMapping(value = "split/err", method = RequestMethod.POST)
    @ResponseBody
    public void splitErr(HttpServletResponse response) {
        ApiUser wu = getApiUser();
        File file = new File(getSubjectSplitErrFile(wu.getId()));
        exportFile("error.txt", file, response);
    }

    private String getSubjectSplitErrFile(Integer userId) {
        String path = tempFile + File.separator + "subject-split-err" + File.separator + userId + File.separator
                + "error.txt";
        return path;
    }

    @ApiOperation(value = "拆分科目-下载模版")
    @RequestMapping(value = "split/template", method = RequestMethod.POST)
    @ResponseBody
    public void splitTemplate(HttpServletResponse response) {
        try {
            List<SubjectSplit> list = new ArrayList<>();
            list.add(new SubjectSplit());
            new ExportExcel("科目拆分数据", SubjectSplit.class, 2).setDataList(list).write(response, "科目拆分导入模板.xlsx")
                    .dispose();
        } catch (Exception e) {
            throw new StatusException("导入模板下载失败！失败信息：" + e.getMessage(), e);
        }
    }
}
